/*************************************************************
 * error.C
 * from Stevens, "Advanced Programming in the Unix Environment"
 *************************************************************/

/*****************************************
 * includes
 *****************************************/
#include <errno.h>
#include "error.H"

void 
dbx_target_stop()
{
}


static void err_doit(int, const char *, va_list);

/*****************************************
 * err_sys:
 * Print a message relating to a 
 * system error, and quit.
 *****************************************/
void err_sys(const char *fmt, ...)
{
   va_list ap;

   va_start(ap, fmt);
   err_doit(1, fmt, ap);
   va_end(ap);
   exit(1);
} /* err_sys */

/*****************************************
 * err_quit:
 * Print a message relating to an error
 * other than a system error, quit.
 *****************************************/
void err_quit(const char *fmt, ...)
{
   va_list ap;

   va_start(ap, fmt);
   err_doit(0, fmt, ap);
   va_end(ap);
   exit(1);
} /* err_quit */

/*****************************************
 * err_ret:
 * Print a message relating to a system
 * error and return.
 *****************************************/
void err_ret(const char *fmt, ...)
{
   va_list ap;

   va_start(ap, fmt);
   err_doit(1, fmt, ap);
   va_end(ap);
   return;
} /* err_ret */

/*****************************************
 * err_msg:
 * Print a message unrelated to a system
 * error and return.
 *****************************************/
void err_msg(const char *fmt, ...)
{
   va_list ap;

   va_start(ap, fmt);
   err_doit(0, fmt, ap);
   va_end(ap);
   return;
} /* err_msg */

/*****************************************
 * err_doit:
 * (Stevens named it, don't blame me).
 * Used in err_sys and err_quit.
 *****************************************/
static void err_doit(int errnoflag,
                    const char *fmt,
                     va_list ap)
{
   int errno_save;
   char buf[MAXLINE];

   errno_save = errno;
   vsprintf(buf, fmt, ap);
   if (errnoflag)
      sprintf(buf+strlen(buf), ": %s", strerror(errno_save));
   strcat(buf, "\n");
   fflush(stdout);
   fputs(buf, stderr);
   fflush(NULL);
   return;
} /* err_doit */


/* end of file error.C */
