// this class contains all of the computational methods for
// computing derivatives of noisy  motion signals
// It will compute and display results.

#ifndef NDIFFTOOLS_H
#define NDIFFTOOLS_H

#include "types.H"
#include "gauss.H"
#include "list.H"

class GlobalWidget;
class Pose;
class Vec3d;
class Quat;

class ErrorVal
{
 public: 
   ErrorVal() : _pvel(0.0,0.0,0.0),
                _pacc(0.0,0.0,0.0),
                _avel(0.0,0.0,0.0,0.0),
                _aacc(0.0,0.0,0.0,0.0)
   { }

 public: 
   Vec3d _pvel;
   Vec3d _pacc;
   Quat  _avel;
   Quat  _aacc;

};


class NumDiff
{
 public:
   NumDiff(GlobalWidget *gbw);
   virtual ~NumDiff();

   void prepare();
   Pose * addNoise(Pose *cpos, int srate, double noisev);

   // computational techniques
   void computeSimpleND();
   Pose * LSOrtho(int win, int deg, double t, FIGarray<Pose*> posewin);
   void computeNDLSOrtho(int win, int deg);
   void computeNDSavGolay(int left, int right, int deg);
   double * savGolay(int left, int right, int deg);
   void computeNDWavelet(int left, int right, int deg,char *wavelet);  
   FIGarray<Pose*> waveletDenoise(FIGarray<Pose*> posewin, int deg,
                                         char *wavelet); 
   void computeNDMollify(int win);
   FIGarray<Pose*> mollify(FIGarray<Pose*> posewin, int win);

   void computeNDABKP(int mode, int win);
   FIGarray<Pose*> ABKP(FIGarray<Pose*> posewin, int win, int mode);

   ErrorVal computeNDRMSE(int win);
 
   // differentiation schemes
   Pose * backwardDiff(FIGarray<Pose*> smlist);
   Pose * centralDiff(FIGarray<Pose*> smlist);
   Pose * diffAvg(FIGarray<Pose*> posewin, int difftype); // 0 for backward
                                                          // 1 for central

   // display functions
   void display(const char *title, int win);
   void drawGraphs();
   void displaySimpleND();
   void displayNDLSOrtho(int win);
   void displayNDSavGolay(int left, int right, int deg);
   void displayNDWavelet(int left, int right, int deg,char *wavelet);
   void displayNDMollify(int win);
   void displayNDABKP(int mode, int win);

   // plot functions
   void plotTCS(char *subplot, char *title, char *xlabel, char *ylabel,
                double *t, double *c, double *s, int numt, int nums);
   void plotVels(char *subplot, char *title, char *xlabel, char *ylabel,
                 double *vt, double *vc, int numt, int nums);
   void velPlotter(int numt, int nums);
   void sigPlotter(int numt, int nums);
 protected:
   GlobalWidget *_gbw;
   GaussWhiteNoise *_posnoise;
   GaussWhiteNoise *_quatnoise;

   FIGarray<Pose*>   _truerec;
   FIGarray<Pose*>   _tmprec;
   FIGarray<Pose*>   _smrec;  // holds new poses after computation
   FIGarray<Pose*>   _posewin; // holds a window of poses

   double _noisev;
   int    _srate;
};


#endif
